function [u, A, b, pt, el] = fem2d_lin(L, H, nx, ny, a, f, g)
%
% Solve 2D boundary value problem with Dirichlet data, i.e, 
%
%   grad^T * (a(x,y) grad u(x,y)) = f(x,y),     (x,y) in D = (0,L) x (0,H),
%
%   u(0,y) = gWest, y in [0,H]
%   u(L,y) = gEast, y in [0,H]
%   u(x,H) = gNorth, x in [0,L]
%   u(x,0) = gSouth, x in [0,L]
%
% The finite element discretization uses P1 shape functions on a
% uniform mesh of (nx-1) x (ny-1) square elements, each divided into two triangles.
% The total number of elements is nel = 2*(nx-1)*(ny-1).
% The total vertices in the mesh is np = nx*ny.
% %
%   2-----1           3
%   |    /           /|
%   | K1/           / |
%   |  /           /  | 
%   | /           / K2|
%   |/           /    |
%   3           1-----2
%
%   INPUT      Description                                         Size
%  
%   L, H       domain [0,L] x [0,H]                                1x1, 1x1
%
%   nx, ny     number of grid points in x- and y-direction         1x1, 1x1
%            
%   a          diffusion coefficient, could be function handle
%              a = @(x,y) ... or string, e.g., 'x+y' to evaluate 
%              a at nodes, or vector of a evaluated of nodes of
%              regular mesh (ordering as in ptable)
%
%   f          sourve term, same format possible as for a
%
%   g          cell array of Dirichlet boundary data               1x4 
%              g = {gWest,gSouth,gEast,gNorth}, gWest and so on
%              can possess same format as a
%
%
%   OUTPUT 
%
%   u          vector of values of Galerkin solution at d.o.f. (nx+1)*(ny+1) 
%
%   A          global stiffness matrix
%
%   b          global load vector
%
%   pt         point table, 2 x np matrix containing coordiantes of mesh
%              vertices
%
%   el         element table, 3 x nel matrix containing global indices of 
%              local nodes of each element
%
%

%% (1) Generate mesh, i.e, element table el and coordinates of vertices pt
[pt,el] = fem2d_lin_mesh(L,H,nx,ny);

% total number of vertices
np = size(pt,2);

% total number of elements (=triangles)
nel = size(el,2);

%% (2) Assembling stiffness matrix A

% Determine input format of diffusion coefficient a
if ischar(a),
    x = pt(1,:)'; y = pt(2,:)';
    a = eval(a);
elseif isa(a,'function_handle')
    x = pt(1,:)'; y = pt(2,:)';
    a = feval(a,x,y);
elseif ~isnumeric(a) || ~min(size(a) == [np,1]),
    error('Diffusionskoeffizient besitzt falsches Format!')
end

% Assemble left-hand side of FE system
%A = fem2d_lin_LHS(a,pt,el); 
A = fem2d_lin_LHS(a,pt,el); 

%% (3) Assemble load vector of FE system.

% Determine input format of source coefficient f
if ischar(f),
    x = pt(1,:)'; y = pt(2,:)';
    f = eval(f);
elseif isa(f,'function_handle')
    x = pt(1,:)'; y = pt(2,:)';
    f = feval(f,x,y);
elseif ~isnumeric(f) || ~min(size(f) == [np,1]),
    error('Quelltermkoeffizient besitzt falsches Format!')
end

% Assemble right-hand side of FE system
b = fem2d_lin_RHS(f,pt,el); 

%% (4) Incorporate essential boundary conditions

% Initialize solution object
u = zeros(np,1);

% Incorporate Dirichlet boundary conditions

% Global indices of nodes at the 4 boundaries
south = 1:nx;
east = nx:nx:np;
north = 1+(ny-1)*nx:np;
west = 1:nx:(np-nx+1);

% Cell-array of these indices
inds{1} = south; inds{2} = east; inds{3} = north; inds{4} = west;

% Loop over 4 boundaries
for i=1:4,
    % Set data and indices according to boudary part
    gi = g{i};
    ind = inds{i};
    
    % Determine input format of Dirichlet boundary data g
    if ischar(gi),
        x = pt(1,ind)'; y = pt(2,ind)';
        gi = eval(gi);
    elseif isa(gi,'function_handle')
        x = pt(1,ind)'; y = pt(2,ind)';
        gi = feval(gi,x,y);
    elseif ~isnumeric(gi) || ~min(size(gi) == [length(ind),1]),
        error(sprintf('Dirichletdaten %i besitzen falsches Format!',i))
    end
    
    % Set corresponding d.o.f. according to boundary data (interpolate data)
    u(ind) = gi;
end

% Correct RHS
b = b - A*u;

% Remove essential d.o.f.s from FE system
[dof0, I0] = setdiff(1:np, [south, east, north, west]);
A0 = A(dof0, dof0);
b0 = b(dof0);

%% (5) Solve FE-System
u0 = A0\b0;
u(I0) = u0;

return

end    

