function A = fem2d_lin_LHS(a,ptable,eltable)
%
% This fucntion assembles the global stiffness matrix
%
% A(i,j) = \int a(x) \grad \phi_i(x) \cdot \grad \phi_j(x) dx
%
% for linear triangle elements on a uniform triangulation of a rectangle:
%
%  10--11--12
%   | / | / |
%   |/  |/  |
%   7---8---9
%   | / | / |
%   |/  |/  |
%   4---5---6
%   | / | / |
%   |/  |/  |
%   1---2---3
%
% Local nodes at upper and lower triangle elements: 
%
%   2-----1           3
%   |    /           /|
%   | K1/           / |
%   |  /           /  | 
%   | /           / K2|
%   |/           /    |
%   3           1-----2
%
% Reference element:
%
%   3
%   |\
%   | \
%   |  \
%   |   \
%   |K   \
%   1-----2
%
% Basis functions on K:
% phi_1(xi, eta) = xi
% phi_2(xi, eta) = eta
% phi_3(xi, eta) = 1 - xi - eta
%
%
%   INPUT      Description                                         Size
%  
%   a          diffusion coefficient evaluated at nodes in         np x 1
%              FE mesh (nodes are ordered lexicographically, 
%              x-direction first)
%
%   pt         matrix containing coordiantes of mesh vertices      2 x np
%
%   el         element table containing global indices of local    3 x nel
%              nodes of each triangle element
%            
%   OUTPUT 
%
%   A         global stiffness matrix (sparse)
%

% Total number of finite elements (=triangles)
nel = size(eltable,2);

% Number of local d.o.f.s per triangle
nloc = size(eltable,1);

% Total number of d.o.f.s (=nodes)
N = size(ptable, 2);

% Mesh widths
hx = abs(ptable(1,2) - ptable(1,1));
nx = max(ptable(1,:))/hx;
hy = abs(ptable(2,nx+2) - ptable(2,1));
ny = max(ptable(2,:))/hy;

% Determinant of matrix B of affine mapping 
% x = B*xi + b from reference to physical to element
detB = hx*hy;       % determinant of B 

% Matrix of gradients of basis functions on physical element
% GradPhi = 1/detB^2 * [hy^2, -hy^2, 0; ...
%                      -hy^2,  hy^2+hx^2, -hx^2;...
%                       0, -hx^2, hx^2];        
                  
% Assembly of Galerkin equations
A = sparse(N,N);

% Integral of a over each element (w.r.t. xi)
int_a = ( a(eltable(1,:)) + a(eltable(2,:)) + a(eltable(3,:)) )'/6;

% Assembling all 9 (resp. 7) local combinations:
% (1,1)
A = A + sparse(eltable(1,:), eltable(1,:), int_a * hy^2 / detB, N, N);
% (1,2)
A = A + sparse(eltable(1,:), eltable(2,:), int_a * (-hy^2) / detB, N, N);
% (2,1)
A = A + sparse(eltable(2,:), eltable(1,:), int_a * (-hy^2) / detB, N, N);
% (2,2)
A = A + sparse(eltable(2,:), eltable(2,:), int_a * (hx^2+hy^2) / detB, N, N);
% (2,3)
A = A + sparse(eltable(2,:), eltable(3,:), int_a * (-hx^2) / detB, N, N);
% (3,2)
A = A + sparse(eltable(3,:), eltable(2,:), int_a * (-hx^2) / detB, N, N);
% (3,3)
A = A + sparse(eltable(3,:), eltable(3,:), int_a * hx^2 / detB, N, N);


end