function b = fem2d_lin_RHS(f,ptable,eltable)
%
% This fucntion assembles the global load vector 
%
% b(i) = \int f(x) \phi_i(x) dx
%
% for linear triangle elements on a uniform triangulation of a rectangle:
%
%  10--11--12
%   | / | / |
%   |/  |/  |
%   7---8---9
%   | / | / |
%   |/  |/  |
%   4---5---6
%   | / | / |
%   |/  |/  |
%   1---2---3
%
% Local nodes at upper and lower triangle elements: 
%
%   2-----1           3
%   |    /           /|
%   | K1/           / |
%   |  /           /  | 
%   | /           / K2|
%   |/           /    |
%   3           1-----2
%
% Reference element:
%
%   3
%   |\
%   | \
%   |  \
%   |   \
%   |K   \
%   1-----2
%
% Basis functions on K:
% phi_1(xi, eta) = xi
% phi_2(xi, eta) = eta
% phi_3(xi, eta) = 1 - xi - eta
%
%
%   INPUT      Description                                         Size
%  
%   f          source term evaluated at nodes in FE mesh           np x 1
%              (nodes are ordered lexicographically, 
%              x-direction first)
%
%   pt         matrix containing coordiantes of mesh vertices      2 x np
%
%   el         element table containing global indices of local    3 x nel
%              nodes of each triangle element
%            
%   OUTPUT 
%
%   b         global load vector (sparse)
%

% Total number of finite elements (=triangles)
nel = size(eltable,2);

% Number of local d.o.f.s per triangle
nloc = size(eltable,1);

% Total number of d.o.f.s (=nodes)
N = size(ptable, 2);

% Mesh widths
hx = abs(ptable(1,2) - ptable(1,1));
m = max(ptable(1,:))/hx;
hy = abs(ptable(2,m+2) - ptable(2,1));
n = max(ptable(2,:))/hy;

% Determinant of matrix B of affine mapping 
% x = B*xi + b from reference to physical to element
detB = hx*hy;       % determinant of B 

% Assembly of load vector
b = sparse(N,1);

% Assembling all 3 local dofs:
for i = 1:nloc,
        b = b + sparse(eltable(i,:)', ones(nel,1), detB/6 * f(eltable(i,:)), N, 1);
end


end