%% Demo script for running an examples for fem2d_lin-code
clear all; close all; clc;

%% (0) Settings
% True solution u(x,y) = sin(pi*x) .* cos(pi/2*y)
u_exa = @(x,y) sin(pi*x) .* y;

% Set domain D = [0,1] x [0,2]
L = 1; % Length
H = 2; % Height

% diffusion term (constant 1)
a = 'y';

% source term
f = 'sin(pi*x) .* (pi^2 * y.^2 - 1)';

% boundary data
g{1} = '0*x';     % Southern boundary data (y=0)
g{2} = '0*y';           % Eastern boundary data (x=L)
g{3} = '2*sin(pi*x)';    % Northern boundary data (y = H)
g{4} = '0*y';           % Western boundary data (x = 0)

%% (1) Compute and plot one solution

% number of rectangular elements per direction
nx = 64; ny = 64;

% generating mesh
[pt, el] = fem2d_lin_mesh(L,H,1+nx,1+ny);

% FEM-solution
u = fem2d_lin(L,H,1+nx,1+ny,a,f,g);

% Plot FEM solution
fem2d_lin_plot(u, pt, el); 
xlabel('x'); ylabel('y');
title('Numerical Solution of elliptic PDE')
set(gca,'FontSize',14)

%% (2) Check Convergence of FEM solutions in L2- and H1-norm

% FEM solutions and their errors for certain mesh widths
hs = 2.^(-(2:5));

% Reference grid for computation of error
h_ref = 2.^(-8);
[pt_ref, el_ref] = fem2d_lin_mesh(L,H,1+nx,1+ny);
x_ref = pt(1,:)'; y_ref = pt(2,:)';

errL2 = zeros(1,length(hs));
errH1 = zeros(1,length(hs));
for i = 1:length(hs),
    % Current mesh width
    h = hs(i);

    % number of rectangular elements per direction
    nx = L/h; ny = H/h;
    
    % generating mesh
    [pt, el] = fem2d_lin_mesh(L,H,1+nx,1+ny);
    
    % Coordinates of nodes in current mesh
    x = pt(1,:)'; y = pt(2,:)';
    
    % FEM-solution
    u = fem2d_lin(L,H,1+nx,1+ny,a,f,g);

    % Error of FEM-solution on fine reference grid
    err = u_exa(x_ref,y_ref) - fem2d_lin_eval([x_ref,y_ref], u, pt, el);

    % L2- und H1-norm of error
    [errL2(i), errH1(i)] = fem2d_lin_norm(err, pt_ref, el_ref);
end

% Plot decay of error
figure();
loglog(hs, errH1,'o-r', hs, errL2,'o-b')
xlabel('Anzahl Rechtecke pro Dimension')
ylabel('Norm der Fehler')
legend('H^1-Fehler','L^2-Fehler')

% Estimate decay of error
g1 = polyfit( log(hs), log(errH1), 1);
g2 = polyfit( log(hs), log(errL2), 1);

hold on;
loglog( hs, exp( g1(1)*log(hs) + g1(2)-1),'--r')
loglog( hs, exp( g2(1)*log(hs) + g2(2)-1),'--b')
hold off;
legend('H^1-Fehler','L^2-Fehler',sprintf('r = %.4f',g1(1)),sprintf('r = %.4f',g2(1)))

