function ips = fem2d_lin_eval(x, u, ptable, eltable)
%
% This function evaluates a FE-element function (linear triangle elements)  
% given by its coefficient vector u (corresponding to values at grid points)
% at certain points x in the rectangular domain
%
% Uniform mesh with lexikographical ordering (x-direction first)
%
%  10--11--12
%   | / | / |
%   |/  |/  |
%   7---8---9
%   | / | / |
%   |/  |/  |
%   4---5---6
%   | / | / |
%   |/  |/  |
%   1---2---3
%
%
%   INPUT      Description                                         Size
%  
%   x          coordinates of points of evaluation                 N x 2
%
%   u          vector of d.o.f. coefficients of FE function        np x 1
%
%   pt         matrix containing coordiantes of mesh vertices      2 x np
%
%   el         element table containing global indices of local    3 x nel
%              nodes of each triangle element
%            
%   OUTPUT 
%
%   ips        values of FE function at x                          np x 1
%

% Computes elementes containing and barycentric coordinates of points x
[ind_el, bary] = tsearchn(ptable', eltable', x);

% Evaluates linear (local) basis functions at barycentric coordinates of points
ips = bary(:,1) .* u(eltable(1,ind_el)) + bary(:,2) .* u(eltable(2,ind_el)) + ...
    bary(:,3) .* u(eltable(3,ind_el));
end

