function [ptable,eltable] = fem2d_lin_mesh(L,H,nx,ny)
%
% This function generates the point and element table for a uniform
% triangulation of the rectangle [0,L]x[0,H] of the form
%
%  10--11--12
%   | / | / |
%   |/  |/  |
%   7---8---9
%   | / | / |
%   |/  |/  |
%   4---5---6
%   | / | / |
%   |/  |/  |
%   1---2---3
%
% with lexikographic global node ordering (as shown).
%
% The local node ordering is as follows for upper (T1) and lower (T2) 
% triangles:
%
%   2-----1           3
%   |    /           /|
%   | T1/           / |
%   |  /           /  | 
%   | /           / T2|
%   |/           /    |
%   3           1-----2
%
%
%   INPUT      Description                                         Size
%  
%   L, H       domain [0,L] x [0,H]                                1x1, 1x1
%
%   nx, ny     number of grid points in x- and y-direction         1x1, 1x1
%
%            
%   OUTPUT 
%
%   pt         point table, 2 x np matrix containing coordiantes of mesh
%              vertices
%
%   el         element table, 3 x nel matrix containing global indices of 
%              local nodes of each element
%
%

hx = L/(nx-1); % mesh width in x-direction
hy = H/(ny-1); % mesh width in y-direction

%% (1) Table containing coordinates of vertices, i.e, d.o.f.s
np = nx*ny;           % Total number of d.o.f.s (=nodes)
ptable = zeros(2,np);

% lexicographic ordering of nodes, x-direction first
x = hx*(0:nx-1);    % discretization in x-direction
y = hy*(0:ny-1);    % discretization in y-direction
[X, Y] = meshgrid(x, y);

X = X'; Y = Y';     % reorder in convenient way

ptable(1,:) = X(:);
ptable(2,:) = Y(:);
 
%% (2) Element table
% Total number of finite elements (=triangles)
nel = 2*(nx-1)*(ny-1);

% Number of local d.o.f.s per triangle
nloc = 3;

eltable = zeros(nloc,nel);

all_nodes = 1:np;

% Vertices at southern, eastern, northern and western boundary
south_nodes = 1:nx;
east_nodes = nx:nx:np;
north_nodes = 1+(ny-1)*nx:np;
west_nodes = 1:nx:np-nx+1;

% Upper triangles T1
eltable(1,1:2:nel-1) = setdiff(all_nodes, [west_nodes, south_nodes]);
eltable(2,1:2:nel-1) = setdiff(all_nodes, [east_nodes, south_nodes]);
eltable(3,1:2:nel-1) = setdiff( all_nodes, [east_nodes, north_nodes] );

% Lower triangles T2
eltable(1,2:2:nel) = eltable(3,1:2:nel-1);
eltable(2,2:2:nel) = setdiff(all_nodes, [west_nodes, north_nodes]);
eltable(3,2:2:nel) = eltable(1,1:2:nel-1);

return
end