function [L2_norm, H1_norm, M, Mgrad] = fem2d_lin_norm(u, ptable, eltable, M, Mgrad)
%
% Computes L2-norm and H1-norm of linear triangle element function given
% by coefficient vector u
%
% This function computes the L^2- and H^1-norm of an FE function (linear 
% triangle elements) given its coefficient vector u
%
% Uniform triangular mesh with lexikographical ordering (x-direction first)
%
%  10--11--12
%   | / | / |
%   |/  |/  |
%   7---8---9
%   | / | / |
%   |/  |/  |
%   4---5---6
%   | / | / |
%   |/  |/  |
%   1---2---3
%
%
%   INPUT      Description                                         Size
%  
%   u          vector of d.o.f. coefficients of FE function        np x 1
%
%   ptable     matrix containing coordiantes of mesh vertices      2 x np
%
%   eltable    element table containing global indices of local    3 x nel
%              nodes of each triangle element
%            
%   M          global mass matrix of FE basis functions          
%              (optinal, sparse or np x np)
%
%   Mgrad      global mass matrix for gradients of FE basis 
%              functions (optinal, sparse or np x np)
%
%   OUTPUT 
%
%   L2norm     L^2 norm of FE function u                           1 x 1
%
%   H1norm     H^1 norm of FE function u                           1 x 1
%
%   M          global mass matrix of FE basis functions            (sparse)
%
%   Mgrad      global mass matrix for gradients of FE basis        (sparse)
%              functions
%

if nargin < 4
    %% Compute global mass matrices

    % Total number of d.o.f.s (=nodes)
    N = size(ptable, 2);

    % Mesh infos
    hx = abs(ptable(1,2) - ptable(1,1));
    nx = max(ptable(1,:))/hx;
    hy = abs(ptable(2,nx+2) - ptable(2,1));

    % Determinant of matrix B of affine mapping 
    % x = B*xi + b from reference to physical to element
    detB = hx*hy;       % determinant of B 

    % Local mass matrix
    M_loc = detB/24 * [2 1 1; 1 2 1; 1 1 2];
    
    % Local mass matrix for gradients of basis functions
    M_grad_loc = 0.5/detB * [hy^2, -hy^2, 0; ...
                             -hy^2,  hy^2+hx^2, -hx^2;...
                              0, -hx^2, hx^2];        
                  

    % Global mass matrix and global mass matrix of gradients
    M = sparse(N,N);
    Mgrad = sparse(N,N);
    for i=1:3,
        for j=1:3,
            M = M + sparse(eltable(i,:), eltable(j,:), M_loc(i,j), N, N);
            Mgrad = Mgrad + sparse(eltable(i,:), eltable(j,:), M_grad_loc(i,j), N, N);
        end
    end
    
    
end

%% Compute norms
L2_norm = sqrt(u' * M * u);
H1_norm = sqrt( L2_norm^2 + u' * Mgrad * u);

end

